unit EXECryptor;

{ EXECryptor API v. 2.1.11 }

{$DEFINE USE_API}
{$DEFINE USE_CRYPT_REG}
{$DEFINE USE_STD_SERIALS}

{$DEFINE ANTIDEBUG_ON_INIT}

interface

uses Windows, SysUtils;

{$IFDEF USE_API}

function SafeGetDate: TDateTime;
function GetHardwareID: DWORD; stdcall;

function EncryptStr(s: string): string;
function DecryptStr(s: string): string;

function SecureWrite(const Name,Value: string): Boolean;
function SecureRead(const Name: string; var Value: string): Boolean;

function SafeMessageBox(hWnd: HWND; lpText, lpCaption: PAnsiChar; uType: UINT): Integer; stdcall;
function SafeGetProcAddr(h: THandle; Name: PChar): TFarProc; stdcall;

procedure AntiDebug; stdcall;
procedure ProtectImport; stdcall;

function GetTrialDaysLeft(TrialPeriod: integer): integer; stdcall;
function GetTrialRunsLeft(TrialRuns: integer): integer; stdcall;

{$ENDIF}

{$IFDEF USE_CRYPT_REG}

type
  TVerifyResult = (vrInvalid, vrExpired, vrOK);

function IsRegistered: TVerifyResult; stdcall;

{$IFDEF USE_STD_SERIALS}

type
  TSerialNumberInfo = packed record
    LicType: integer;    //0..15
    // if LicType = 1 then we get ExpiryMonth/ExpiryYear
    // otherwise we get UserParam
    UserParam: integer;  //0..1023
    ExpiryMonth,         //1..12
    ExpiryYear: integer; //2004..2024
    F1,F2,F3,F4,F5,F6: Boolean;
  end;
  
function VerifySerialNumber(const RegistrationName,
  SerialNumber: string; const HardwareID: string = ''): TVerifyResult;

function VerifySerialNumberEx(const RegistrationName,
  SerialNumber: string; var SerialNumberInfo: TSerialNumberInfo;
  const HardwareID: string = ''): TVerifyResult;

{$ELSE}

procedure SetCodeKey(Key: Pointer; Size: integer); stdcall;

{$ENDIF}

function RegConst_0: DWORD; stdcall;
function RegConst_1: DWORD; stdcall;
function RegConst_2: DWORD; stdcall;
function RegConst_3: DWORD; stdcall;
function RegConst_4: DWORD; stdcall;
function RegConst_5: DWORD; stdcall;
function RegConst_6: DWORD; stdcall;
function RegConst_7: DWORD; stdcall;

{$ENDIF}

// need for correct entry point protection
procedure EXECryptor_halt;

implementation

{$O-}
{$IFDEF USE_API}
function GetHardwareID: DWORD; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=0;
end;

function EXECryptor_GetDate: DWORD; stdcall;
var t1,t2: TFileTime;
    dt,tt: WORD;
begin
  GetSystemTimeAsFileTime(t1);
  FileTimeToLocalFileTime(t1,t2);
  FileTimeToDosDateTime(t2,dt,tt);
  Result:=dt;
end;

function SafeGetDate: TDateTime;
var dt: DWORD;
begin
  dt:=EXECryptor_GetDate;
  try
    Result:=EncodeDate((dt shr 9)+1980,(dt shr 5) and $F,dt and $1F);
  except
    Result:=0;
  end;
end;

procedure EXECryptor_EncryptStr(Src,Dst: PChar); stdcall;
begin
  StrCopy(Dst,Src);
end;

function EncryptStr(s: string): string;
begin
  SetLength(Result,Length(s)*2+4);
  EXECryptor_EncryptStr(PChar(s),PChar(Result));
  Result:=PChar(Result);
end;

procedure EXECryptor_DecryptStr(Src,Dst: PChar); stdcall;
begin
  StrCopy(Dst,Src);
end;

function DecryptStr(s: string): string;
begin
  SetLength(Result,Length(s));
  EXECryptor_DecryptStr(PChar(s),PChar(Result));
  Result:=PChar(Result);
end;

const
  TestKeyPath = 'Software\EXECryptorTestKeys';

function EXECryptor_SecureWrite(Name,Value: PChar): Boolean; stdcall;
var Disposition: integer;
    KeyHandle: HKEY;
begin
  Result:=False;
  if RegCreateKeyExA(HKEY_CURRENT_USER, TestKeyPath, 0, nil,
      REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, nil, KeyHandle, @Disposition) = ERROR_SUCCESS then try
    RegSetValueExA(KeyHandle, Name, 0, REG_BINARY, PByte(Value), strlen(Value)+1);
    Result:=True;
  finally
    RegCloseKey(KeyHandle);
  end;
end;

function SecureWrite(const Name,Value: string): Boolean;
begin
  Result:=EXECryptor_SecureWrite(PChar(Name),PChar(Value));
end;

function EXECryptor_SecureRead(Name,Value: PChar): Boolean; stdcall;
var KeyHandle: HKEY;
    sz,tp: DWORD;
begin
  Result:=False;
  if RegOpenKeyExA(HKEY_CURRENT_USER, TestKeyPath, 0, KEY_QUERY_VALUE, KeyHandle) = ERROR_SUCCESS then try
    tp:=REG_BINARY;
    if RegQueryValueExA(KeyHandle, Name, nil, @tp, nil, @sz) = ERROR_SUCCESS then begin
      RegQueryValueExA(KeyHandle, Name, nil, @tp, PByte(Value), @sz);
      Result:=True;
    end;
  finally
    RegCloseKey(KeyHandle);
  end;
end;

function SecureRead(const Name: string; var Value: string): Boolean;
begin
  SetLength(Value,1024);
  Result:=EXECryptor_SecureRead(PChar(Name),PChar(Value));
  if Result then Value:=PChar(Value)
  else Value:='';
end;

function SafeMessageBox(hWnd: HWND; lpText, lpCaption: PAnsiChar; uType: UINT): Integer; stdcall;
begin
  Result:=MessageBoxA(hWnd,lpText,lpCaption,uType);
end;

function SafeGetProcAddr(h: THandle; Name: PChar): TFarProc; stdcall;
begin
  Result:=GetProcAddress(h,Name);
end;

procedure AntiDebug; stdcall;
begin
  asm
    db $90,$90,$90,$90,$90
  end;
end;

procedure ProtectImport; stdcall;
begin
  asm
    db $90,$90,$90,$90,$90
  end;
end;

function GetTrialDaysLeft(TrialPeriod: integer): integer; stdcall;
begin
  Result:=TrialPeriod;
end;

function GetTrialRunsLeft(TrialRuns: integer): integer; stdcall;
begin
  Result:=TrialRuns;
end;

{$ENDIF}

{$IFDEF USE_CRYPT_REG}

function IsRegistered: TVerifyResult; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=vrOK;
end;

{$IFDEF USE_STD_SERIALS}

type
  PSerialNumberInfo = ^TSerialNumberInfo;

function EXECryptor_VerifySerialNumber(RegName,SN: PChar;
  CurrYear, CurrMonth: integer;
  SNInfo: PSerialNumberInfo; HWID: PChar): integer; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=2;//vrOK;
end;

function VerifySerialNumber(const RegistrationName,
  SerialNumber: string; const HardwareID: string): TVerifyResult;
var
  CurrYear, CurrMonth, d: WORD;
begin
  {$I crypt_start.inc}
{$IFDEF USE_API}
  DecodeDate(SafeGetDate,CurrYear, CurrMonth, d);
{$ELSE}
  DecodeDate(Date,CurrYear, CurrMonth, d);
{$ENDIF}
  Result:=TVerifyResult(EXECryptor_VerifySerialNumber(PChar(RegistrationName),
      PChar(SerialNumber),CurrYear, CurrMonth,nil,PChar(HardwareID)));
  {$I crypt_end.inc}
end;

function VerifySerialNumberEx(const RegistrationName,
  SerialNumber: string; var SerialNumberInfo: TSerialNumberInfo;
  const HardwareID: string): TVerifyResult;
var
  CurrYear, CurrMonth, d: WORD;
begin
  {$I crypt_start.inc}
{$IFDEF USE_API}
  DecodeDate(SafeGetDate,CurrYear, CurrMonth, d);
{$ELSE}
  DecodeDate(Date,CurrYear, CurrMonth, d);
{$ENDIF}
  Result:=TVerifyResult(EXECryptor_VerifySerialNumber(PChar(RegistrationName),
      PChar(SerialNumber),CurrYear, CurrMonth,@SerialNumberInfo,PChar(HardwareID)));
  {$I crypt_end.inc}
end;

{$ELSE}

procedure SetCodeKey(Key: Pointer; Size: integer); stdcall;
begin
  asm
    db $90,$90,$90,$90,$90
  end;
end;
{$ENDIF}

function RegConst_0: DWORD; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=0;
end;

function RegConst_1: DWORD; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=1;
end;

function RegConst_2: DWORD; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=2;
end;

function RegConst_3: DWORD; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=3;
end;

function RegConst_4: DWORD; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=4;
end;

function RegConst_5: DWORD; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=5;
end;

function RegConst_6: DWORD; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=6;
end;

function RegConst_7: DWORD; stdcall;
begin
  asm
    db $90,$90,$90,$90
  end;
  Result:=7;
end;
{$ENDIF}
{$O+}

procedure EXECryptor_halt;
begin
  Halt;
end;

{$IFDEF USE_CRYPT_REG}
exports
  IsRegistered name 'EXECryptor_IsRegistered';
{$IFDEF USE_STD_SERIALS}
exports
  EXECryptor_VerifySerialNumber;
{$ELSE}
exports
  SetCodeKey name 'EXECryptor_SetCodeKey';
{$ENDIF}
exports
  RegConst_0 name 'EXECryptor_RegConst_0',
  RegConst_1 name 'EXECryptor_RegConst_1',
  RegConst_2 name 'EXECryptor_RegConst_2',
  RegConst_3 name 'EXECryptor_RegConst_3',
  RegConst_4 name 'EXECryptor_RegConst_4',
  RegConst_5 name 'EXECryptor_RegConst_5',
  RegConst_6 name 'EXECryptor_RegConst_6',
  RegConst_7 name 'EXECryptor_RegConst_7';
{$ENDIF}
{$IFDEF USE_API}
exports
  GetHardwareID name 'EXECryptor_GetHardwareID',
  EXECryptor_GetDate,
  EXECryptor_EncryptStr,
  EXECryptor_DecryptStr,
  EXECryptor_SecureWrite,
  EXECryptor_SecureRead,
  SafeMessageBox name 'EXECryptor_MessageBox',
  SafeGetProcAddr name 'EXECryptor_GetProcAddr',
  AntiDebug name 'EXECryptor_AntiDebug',
  ProtectImport name 'EXECryptor_ProtectImport',
  GetTrialDaysLeft name 'EXECryptor_GetTrialDaysLeft',
  GetTrialRunsLeft name 'EXECryptor_GetTrialRunsLeft';
{$ENDIF}
exports
  EXECryptor_halt name 'EXECryptor_halt';
                      
{$IFDEF ANTIDEBUG_ON_INIT}
{$IFDEF USE_API}
initialization
  {$I crypt_start.inc}
  ProtectImport;
  AntiDebug;
  {$I crypt_end.inc}
{$ENDIF}
{$ENDIF}
end.
